﻿using System;
using System.Configuration;
using System.Net;
using System.Web.Mvc;
using MvcErrorHandling.DomainModel.AppSettingsModel;

namespace MvcErrorHandling.Controllers
{
    public class ErrorController : Controller
    {
        [HttpGet]
        public ActionResult CustomErrors403()
        {
            return CustomErrorsCore((int)HttpStatusCode.Forbidden);
        }

        [HttpGet]
        public ActionResult CustomErrors404()
        {
            return CustomErrorsCore((int)HttpStatusCode.NotFound);
        }

        [HttpGet]
        public ActionResult CustomErrors500()
        {
            return CustomErrorsCore((int)HttpStatusCode.InternalServerError);
        }

        [HttpGet]
        public ActionResult HttpErrors403()
        {
            return HttpErrorsCore((int)HttpStatusCode.Forbidden);
        }

        [HttpGet]
        public ActionResult HttpErrors404()
        {
            return HttpErrorsCore((int)HttpStatusCode.NotFound);
        }

        [HttpGet]
        public ActionResult HttpErrors500()
        {
            return HttpErrorsCore((int)HttpStatusCode.InternalServerError);
        }

        public ActionResult ManualErrors(int? statusCode)
        {
            if (statusCode == null)
            {
                if (HttpContext.Session != null && HttpContext.Session["statusCode"] != null)
                {
                    statusCode = (int)HttpContext.Session["statusCode"];
                }
            }

            var exception = RouteData.Values["exception"] as Exception;

            if (exception == null && HttpContext.Session != null)
            {
                exception = HttpContext.Session["exception"] as Exception;
            }

            if (exception == null)
            {
                exception = Server.GetLastError();
            }

            if (statusCode == default(int))
            {
                statusCode = (int)HttpStatusCode.InternalServerError;
            }

            return ManualErrorsCore(statusCode ?? 999, exception);
        }

        private ActionResult CustomErrorsCore(int httpStatusCode)
        {
            AllErrorsCore(httpStatusCode);

            ViewBag.ExceptionMessage = GetExceptionMessage();
            ViewBag.AspxErrorPath = GetAspxErrorPath();

            return View("CustomErrors");
        }

        private ActionResult HttpErrorsCore(int httpStatusCode)
        {
            AllErrorsCore(httpStatusCode);

            ViewBag.ExceptionMessage = GetExceptionMessage();

            return View("HttpErrors");
        }

        private ActionResult ManualErrorsCore(int httpStatusCode, Exception exception)
        {
            AllErrorsCore(httpStatusCode);

            ViewBag.ExceptionMessage = exception != null ? exception.Message : "The exception information is not available.";

            return View("ManualErrors");
        }

        private void AllErrorsCore(int httpStatusCode)
        {
            Response.ClearHeaders();
            Response.ClearContent();
            Response.StatusCode = httpStatusCode;

            if (ConfigurationManager.AppSettings.Include(TrySkipIisCustomErrorsMode.Skip))
            {
                Response.TrySkipIisCustomErrors = true;
            }

            ViewBag.SessionData = MvcApplication.GetSessionData();
            ViewBag.StatusCode = Response.StatusCode.ToString();
        }

        private string GetExceptionMessage()
        {
            var exception = Server.GetLastError();

            return exception != null ? exception.Message : "The exception information is not available.";
        }

        private string GetAspxErrorPath()
        {
            var aspxErrorPath = Request.QueryString["aspxerrorpath"];

            return string.IsNullOrEmpty(aspxErrorPath) ? "The aspxerrorpath parameter is not available." : aspxErrorPath;
        }
    }
}
