﻿using System.IO;
using System.Net;
using System.Web;
using System.Web.Mvc;
using MvcErrorHandling.DomainModel;
using MvcErrorHandling.MvcHelpers;

namespace MvcErrorHandling.Controllers
{
    public class ErrorGenerationController : Controller
    {
        [HttpGet]
        public ActionResult UnhandledException()
        {
            throw new TestException("This exception was deliberately thrown by a GET action method.");
        }

        [HttpPost]
        public ActionResult UnhandledExceptionPost()
        {
            throw new TestException("This exception was deliberately thrown by a POST action method.");
        }

        [HttpGet]
        public ActionResult UnhandledExceptionWithValidationError(string inputText)
        {
            return null; // This point will never be reached.
        }

        [HttpGet]
        [Ajax(true)]
        public ActionResult UnhandledExceptionAjax()
        {
            throw new TestException("This exception was deliberately thrown by an AJAX action method.");
        }

        [HttpGet]
        public ActionResult NotFoundException()
        {
            throw new HttpException(
                (int)HttpStatusCode.NotFound,
                "This 404 exception was deliberately thrown from an action method.");
        }

        [HttpGet]
        [Ajax(true)]
        public ActionResult NotFoundExceptionAjax()
        {
            throw new HttpException(
                (int)HttpStatusCode.NotFound,
                "This 404 exception was deliberately thrown from an AJAX action method.");
        }

        [HttpGet]
        public ActionResult FailedFileDownload()
        {
            return new FileStreamResult(new UnreadableStream(), "image/jpeg");
        }

        private class UnreadableStream : Stream
        {
            public override int Read(byte[] buffer, int offset, int count)
            {
                throw new TestException("This exception was deliberately thrown from within a FileStreamResult.");
            }

            public override void Flush() { }
            public override long Seek(long offset, SeekOrigin origin) { return offset; }
            public override void SetLength(long value) { }
            public override void Write(byte[] buffer, int offset, int count) {}
            public override bool CanRead { get { return false; } }
            public override bool CanSeek { get { return false; } }
            public override bool CanWrite { get { return false; } }
            public override long Length { get { return 1; } }
            public override long Position { get; set; }
        }
    }
}
