﻿using System;
using System.Web.Mvc;
using AutoMapper;
using MvcErrorHandling.DomainModel;
using MvcErrorHandling.Models.Home;
using MvcErrorHandling.MvcHelpers;
using MvcErrorHandling.Services;

namespace MvcErrorHandling.Controllers
{
    public class HomeController : Controller
    {
        private readonly ErrorConfigurationService _errorConfigurationService;

        public HomeController()
        {
            _errorConfigurationService = new ErrorConfigurationService();
        }

        [HttpGet]
        public ActionResult Index()
        {
            var viewModel = ReadErrorConfiguration();

            return View(viewModel);
        }

        [HttpPost]
        [Ajax(false)]
        public ActionResult Index(IndexViewModel viewModel)
        {
            if (!ModelState.IsValid)
            {
                return View(viewModel);
            }

            WriteErrorConfiguration(viewModel);

            return Redirect("/");
        }

        [HttpPost]
        [Ajax(true)]
        [ActionName("Index")]
        public ActionResult IndexViaAjax(IndexViewModel viewModel)
        {
            if (ModelState.IsValid)
            {
                WriteErrorConfiguration(viewModel);
            }

            return PartialView("EditorTemplates/IndexViewModel", viewModel);
        }

        [HttpPost]
        [Ajax(false)]
        public ActionResult Reset()
        {
            _errorConfigurationService.ResetErrorConfiguration();

            return Redirect("/");
        }

        [HttpPost]
        [Ajax(true)]
        [ActionName("Reset")]
        public ActionResult ResetViaAjax()
        {
            _errorConfigurationService.ResetErrorConfiguration();

            var viewModel = ReadErrorConfiguration();

            return PartialView("EditorTemplates/IndexViewModel", viewModel);
        }

        private void WriteErrorConfiguration(IndexViewModel viewModel)
        {
            var errorConfigurationView = Mapper.Map<IndexViewModel, ErrorConfigurationView>(viewModel);

            _errorConfigurationService.WriteErrorConfiguration(errorConfigurationView);
        }

        private IndexViewModel ReadErrorConfiguration()
        {
            ErrorConfigurationView errorConfigurationView;
            try
            {
                errorConfigurationView = _errorConfigurationService.ReadErrorConfiguration();
            }
            catch (Exception)
            {
                // Trying this to make the app more resilient to times when the Web.config file gets out of wack.
                _errorConfigurationService.ResetErrorConfiguration();
                errorConfigurationView = _errorConfigurationService.ReadErrorConfiguration();
            }

            var viewModel = Mapper.Map<ErrorConfigurationView, IndexViewModel>(errorConfigurationView);

            return viewModel;
        }
    }
}
