﻿using System;
using System.Collections.Specialized;
using System.Configuration;

namespace MvcErrorHandling.DomainModel.AppSettingsModel
{
    public static class AppSettingsExtensions
    {
        public static TEnum GetEnumSetting<TEnum>(this KeyValueConfigurationCollection collection)
        {
            var key = typeof (TEnum).Name;

            var valueElement = collection[key];

            if (valueElement == null || string.IsNullOrWhiteSpace(valueElement.Value))
            {
                var log = string.Format("Failed to find AppSetting for key {0}.", key);
                throw new Exception(log);
            }

            var value = (TEnum) Enum.Parse(typeof (TEnum), valueElement.Value, true);

            return value;
        }

        public static void UpdateEnumSetting<TEnum>(this KeyValueConfigurationCollection collection, TEnum newValue)
        {
            var key = typeof (TEnum).Name;
            var value = newValue.ToString();

            if (collection[key] != null)
            {
                collection.Remove(key);
            }

            collection.Add(key, value);
        }

        public static bool Include<TEnum>(this NameValueCollection collection, TEnum valueToTest)
        {
            return collection[typeof (TEnum).Name] == valueToTest.ToString();
        }
    }
}