﻿using System;
using System.Configuration;

namespace MvcErrorHandling.DomainModel.AppSettingsModel
{
    public class AppSettingsWriter
    {
        private readonly KeyValueConfigurationCollection _appSettings;

        public ErrorTriggerLinkOpenMode ErrorTriggerLinkOpenMode { get; private set; }

        public ManualServerErrorHandlingMode ManualServerErrorHandlingMode { get; private set; }

        public TrySkipIisCustomErrorsMode TrySkipIisCustomErrorsMode { get; private set; }

        public ErrorPageRenderingMode ErrorPageRenderingMode { get; private set; }

        public ErrorPageExceptionsMode ErrorPageExceptionsMode { get; private set; }

        public ManualNotFoundErrorHandlingMode ManualNotFoundErrorHandlingMode { get; private set; }

        public ManualErrorsRenderingMode ManualErrorsRenderingMode { get; private set; }

        public AppSettingsWriter(KeyValueConfigurationCollection appSettings)
        {
            if (appSettings == null) throw new ArgumentNullException("appSettings");

            _appSettings = appSettings;
        }

        public void ReadSettings()
        {
            ErrorTriggerLinkOpenMode = _appSettings.GetEnumSetting<ErrorTriggerLinkOpenMode>();
            ManualServerErrorHandlingMode = _appSettings.GetEnumSetting<ManualServerErrorHandlingMode>();
            TrySkipIisCustomErrorsMode = _appSettings.GetEnumSetting<TrySkipIisCustomErrorsMode>();
            ErrorPageRenderingMode = _appSettings.GetEnumSetting<ErrorPageRenderingMode>();
            ErrorPageExceptionsMode = _appSettings.GetEnumSetting<ErrorPageExceptionsMode>();
            ManualNotFoundErrorHandlingMode = _appSettings.GetEnumSetting<ManualNotFoundErrorHandlingMode>();
            ManualErrorsRenderingMode = _appSettings.GetEnumSetting<ManualErrorsRenderingMode>();
        }

        public void UpdateSettings()
        {
            UpdateSettings(
                EnumHelpers.GetDefaultEnumValue<ErrorTriggerLinkOpenMode>(),
                EnumHelpers.GetDefaultEnumValue<ManualServerErrorHandlingMode>(),
                EnumHelpers.GetDefaultEnumValue<TrySkipIisCustomErrorsMode>(),
                EnumHelpers.GetDefaultEnumValue<ErrorPageRenderingMode>(),
                EnumHelpers.GetDefaultEnumValue<ErrorPageExceptionsMode>(),
                EnumHelpers.GetDefaultEnumValue<ManualNotFoundErrorHandlingMode>(),
                EnumHelpers.GetDefaultEnumValue<ManualErrorsRenderingMode>());
        }

        public void UpdateSettings(
            ErrorTriggerLinkOpenMode errorTriggerLinkOpenMode,
            ManualServerErrorHandlingMode manualServerErrorHandlingMode,
            TrySkipIisCustomErrorsMode trySkipIisCustomErrorsMode,
            ErrorPageRenderingMode errorPageRenderingMode,
            ErrorPageExceptionsMode errorPageExceptionsMode,
            ManualNotFoundErrorHandlingMode manualNotFoundErrorHandlingMode,
            ManualErrorsRenderingMode manualErrorsRenderingMode)
        {
            ErrorTriggerLinkOpenMode = errorTriggerLinkOpenMode;
            _appSettings.UpdateEnumSetting(ErrorTriggerLinkOpenMode);

            ManualServerErrorHandlingMode = manualServerErrorHandlingMode;
            _appSettings.UpdateEnumSetting(ManualServerErrorHandlingMode);

            TrySkipIisCustomErrorsMode = trySkipIisCustomErrorsMode;
            _appSettings.UpdateEnumSetting(TrySkipIisCustomErrorsMode);

            ErrorPageRenderingMode = errorPageRenderingMode;
            _appSettings.UpdateEnumSetting(ErrorPageRenderingMode);

            ErrorPageExceptionsMode = errorPageExceptionsMode;
            _appSettings.UpdateEnumSetting(ErrorPageExceptionsMode);

            ManualNotFoundErrorHandlingMode = manualNotFoundErrorHandlingMode;
            _appSettings.UpdateEnumSetting(ManualNotFoundErrorHandlingMode);

            ManualErrorsRenderingMode = manualErrorsRenderingMode;
            _appSettings.UpdateEnumSetting(ManualErrorsRenderingMode);
        }
    }
}