﻿using System.Linq;
using System.Web.Configuration;

namespace MvcErrorHandling.DomainModel.CustomErrorsModel
{
    public class CustomErrors
    {
        private readonly CustomErrorsSection _customErrorsSection;
        private readonly CustomErrorsError _notFoundError = new CustomErrorsError(404);
        private readonly CustomErrorsError _forbiddenError = new CustomErrorsError(403);
        private readonly CustomErrorsError _serverError = new CustomErrorsError(500);

        public CustomErrorsMode Mode { get; private set; }
        public CustomErrorsRedirectMode RedirectMode { get; private set; }
        public CustomErrorsRedirectType NotFoundErrorRedirect { get; private set; }
        public CustomErrorsRedirectType ForbiddenErrorRedirect { get; private set; }
        public CustomErrorsRedirectType ServerErrorRedirect { get; private set; }

        public CustomErrors(CustomErrorsSection customErrorsSection)
        {
            _customErrorsSection = customErrorsSection;
        }

        public void ReadSection()
        {
            Mode = _customErrorsSection.Mode.FromSystemCustomErrorsMode();
            RedirectMode = _customErrorsSection.RedirectMode.FromSystemCustomErrorsRedirectMode();
            NotFoundErrorRedirect = _notFoundError.ParseCustomErrorsRedirectType(_customErrorsSection.Errors);
            ForbiddenErrorRedirect = _forbiddenError.ParseCustomErrorsRedirectType(_customErrorsSection.Errors);
            ServerErrorRedirect = _serverError.ParseCustomErrorsRedirectType(_customErrorsSection.Errors);
        }

        public void UpdateSection()
        {
            UpdateSection(
                EnumHelpers.GetDefaultEnumValue<CustomErrorsMode>(),
                EnumHelpers.GetDefaultEnumValue<CustomErrorsRedirectMode>(),
                EnumHelpers.GetDefaultEnumValue<CustomErrorsRedirectType>(),
                EnumHelpers.GetDefaultEnumValue<CustomErrorsRedirectType>(),
                EnumHelpers.GetDefaultEnumValue<CustomErrorsRedirectType>());
        }
                
        public void UpdateSection(
            CustomErrorsMode mode,
            CustomErrorsRedirectMode redirectMode,
            CustomErrorsRedirectType notFoundErrorRedirect,
            CustomErrorsRedirectType forbiddenErrorRedirect,
            CustomErrorsRedirectType serverErrorRedirect)
        {
            Mode = mode;
            RedirectMode = redirectMode;
            NotFoundErrorRedirect = notFoundErrorRedirect;
            ForbiddenErrorRedirect = forbiddenErrorRedirect;
            ServerErrorRedirect = serverErrorRedirect;

            _customErrorsSection.Mode = Mode.ToSystemCustomErrorsMode();
            _customErrorsSection.RedirectMode = RedirectMode.ToSystemCustomErrorsRedirectMode();

            SetCustomErrorsSectionErrors(
                _notFoundError.CreateCustomError(NotFoundErrorRedirect),
                _forbiddenError.CreateCustomError(ForbiddenErrorRedirect),
                _serverError.CreateCustomError(ServerErrorRedirect));
        }

        private void SetCustomErrorsSectionErrors(params CustomError[] customErrors)
        {
            _customErrorsSection.Errors.Clear();

            foreach (var customError in customErrors.Where(x => x != null))
            {
                _customErrorsSection.Errors.Add(customError);
            }
        }
    }
}