﻿using System;
using System.Web.Configuration;

namespace MvcErrorHandling.DomainModel.CustomErrorsModel
{
    public class CustomErrorsError
    {
        private const string StaticPageFormatString = "~/CustomErrors{0}.htm";
        private const string MvcPageFormatString = "~/Error/CustomErrors{0}";
        private const string AspxPageFormatString = "~/CustomErrors{0}.aspx";

        private readonly int _statusCode;

        public CustomErrorsError(int statusCode)
        {
            _statusCode = statusCode;
        }

        public CustomErrorsRedirectType ParseCustomErrorsRedirectType(CustomErrorCollection customErrorCollection)
        {
            var customError = customErrorCollection[_statusCode.ToString()];

            if (customError == null)
            {
                return EnumHelpers.GetDefaultEnumValue<CustomErrorsRedirectType>();
            }

            var htmPage = string.Format(StaticPageFormatString, _statusCode);
            var mvcPage = string.Format(MvcPageFormatString, _statusCode);
            var aspxPage = string.Format(AspxPageFormatString, _statusCode);

            if (string.Equals(customError.Redirect, htmPage, StringComparison.OrdinalIgnoreCase))
            {
                return CustomErrorsRedirectType.StaticErrorPage;
            }

            if (string.Equals(customError.Redirect, mvcPage, StringComparison.OrdinalIgnoreCase))
            {
                return CustomErrorsRedirectType.MvcErrorPage;
            }

            if (string.Equals(customError.Redirect, aspxPage, StringComparison.OrdinalIgnoreCase))
            {
                return CustomErrorsRedirectType.AspxErrorPage;
            }

            throw new Exception(
                string.Format("Failed to match redirect {0} for status code {1}.", customError.Redirect, _statusCode));
        }

        public CustomError CreateCustomError(CustomErrorsRedirectType customErrorsRedirectType)
        {
            if (customErrorsRedirectType == CustomErrorsRedirectType.NoPage)
            {
                return null;
            }

            string formatString;

            switch (customErrorsRedirectType)
            {
                case CustomErrorsRedirectType.StaticErrorPage:
                    formatString = StaticPageFormatString;
                    break;
                case CustomErrorsRedirectType.MvcErrorPage:
                    formatString = MvcPageFormatString;
                    break;
                case CustomErrorsRedirectType.AspxErrorPage:
                    formatString = AspxPageFormatString;
                    break;
                default:
                    throw new ArgumentOutOfRangeException("customErrorsRedirectType");
            }

            var redirect = string.Format(formatString, _statusCode);

            return new CustomError(_statusCode, redirect);
        }
    }
}