﻿using System.Collections.Generic;

namespace MvcErrorHandling.DomainModel.ErrorLinksModel
{
    public class ErrorLinkRepository
    {
        private const string AjaxLinkClass = "ajax-error-link";

        public static ErrorLinkGroup FourHundredErrorLinks { get; private set; }
        public static ErrorLinkGroup FiveHundredErrorLinks { get; private set; }

        static ErrorLinkRepository()
        {
            FourHundredErrorLinks = new ErrorLinkGroup(
                "404 ERRORS",
                new List<ErrorLink>
                    {
                        new ErrorLink("~/ErrorGeneration/NotFoundException", "Action method 404 exception",
                                      "An action method that throws a HttpException with a status code of 404."),
                        new ErrorLink("~/ErrorGeneration/NotFoundExceptionAjax", "JSON action method 404 exception",
                                      "An AJAX action method that throws a HttpException with a status code of 404.", AjaxLinkClass),
                        new ErrorLink("~/ErrorGeneration/UnknownAction", "GET for unknown action",
                                      "A GET request where the controller is known but the action method is not."),
                        new ErrorLink("~/ErrorGeneration/UnknownActionPost", "POST for unknown action",
                                      "A POST request where the controller is known but the action method is not.", "post-link"),
                        new ErrorLink("~/UnknownController/Whatever", "Unknown controller",
                                      "The controller is unknown."),
                        new ErrorLink("~/ErrorGeneration/UnhandledException/Whatever/Whatever", "Does not match any route",
                                      "The path does not match any registered route."),
                        
                        new ErrorLink("~/con", "Zombie DOS",
                                      "A zombie DOS path. (The other such paths are com1 to com9, lpt1 to lpt9, aux, prt and nul.)"),
                        
                        new ErrorLink("~/app_code", "app_code directory",
                                      "An ASP.NET directory that does not exist on this site"),
                        new ErrorLink("~/bin", "bin directory",
                                      "An ASP.NET directory that exists but that should not be served."),
                        new ErrorLink("~/Web.config", "Web.config",
                                      "An ASP.NET file that exists but that should not be served."),
                        new ErrorLink("~/global.asax", "global.asax",
                                      "An ASP.NET file that exists but that should not be served."),

                        new ErrorLink("~/controllers", "Controllers directory",
                                      "An MVC directory that exists but that should not be served."),
                        new ErrorLink("~/views", "Views directory",
                                      "An MVC directory that exists but that should not be served."),
                        new ErrorLink("~/views/home/index.cshtml", "Razor view file",
                                      "A Razor view file that exists but that should not be served."),
                        new ErrorLink("~/views/shared/_layout.cshtml", "Razor layout file",
                                      "A Razor file that starts with an underscore, creating a pseudo-404 error."),
                        new ErrorLink("~/content", "Content directory",
                                      "A directory that should not be browseable."),
                        new ErrorLink("~/unknown.jpg", "Unknown image", "An image that does not exist."),
                    });

            FiveHundredErrorLinks = new ErrorLinkGroup(
                "500 ERRORS",
                new List<ErrorLink>
                    {
                        new ErrorLink(
                            "~/ErrorGeneration/UnhandledException",
                            "Action method exception on GET request",
                            "A GET request to an action method that throws an unhandled exception."),

                        new ErrorLink(
                            "~/ErrorGeneration/UnhandledExceptionPost",
                            "Action method exception on POST request",
                            "A POST request to an action method that throws an unhandled exception.",
                            "post-link"),

                        new ErrorLink(
                            "~/ErrorGeneration/UnhandledExceptionAjax",
                            "Action method exception on AJAX request",
                            "An AJAX request to an action method that throws an unhandled exception.",
                            AjaxLinkClass),

                        new ErrorLink(
                            "~/ErrorGeneration/FailedFileDownload",
                            "Failed file download",
                            "A request for a file where the FileStreamResult throws an unhandled exception."),

                        new ErrorLink(
                            "~/ErrorGeneration/UnhandledExceptionWithValidationError?inputText=<s",
                            "Request validation error",
                            "A request that fails because the query string contains a potentially dangerous value."),

                        new ErrorLink(
                            "~/ErrorGeneration/BeginRequestException",
                            "BeginRequest exception",
                            "A request that fails because the BeginRequest event throws an unhandled exception.")
                    });
        }
    }
}
