﻿using System;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using AutoMapper;
using MvcErrorHandling.DomainModel;
using MvcErrorHandling.MvcHelpers;

namespace MvcErrorHandling
{
    // ReSharper disable InconsistentNaming
    public class MvcApplication : HttpApplication
    {
        public static string SessionDataKey = "SessionData";

        public static void RegisterGlobalFilters(GlobalFilterCollection filters)
        {
            filters.Add(
                new SkippableHandleErrorAttribute
                    {
                        View = "HandleErrorAttributeMvcErrorPage"
                    });
        }

        public static void RegisterRoutes(RouteCollection routes)
        {
            routes.IgnoreRoute("{resource}.axd/{*pathInfo}");

            routes.MapRoute(
                "Default",
                "{controller}/{action}/{id}",
                new {controller = "Home", action = "Index", id = UrlParameter.Optional});

            routes.MapRoute(
                "Catch all",
                "{*url}",
                new {controller = "Error", action = "ManualErrors404"},
                new {Constraint = new SkippableRouteConstraint()});
        }

        protected void Application_Start()
        {
            ControllerBuilder.Current.SetControllerFactory(new NotFoundHandlerControllerFactory());

            System.Web.WebPages.Razor.WebPageRazorHost.AddGlobalImport("MvcErrorHandling");

            Mapper.Initialize(x => x.AddProfile<AutoMapperProfile>());

            AreaRegistration.RegisterAllAreas();

            RegisterGlobalFilters(GlobalFilters.Filters);

            RegisterRoutes(RouteTable.Routes);

            ModelMetadataProviders.Current = new CustomDataAnnotationsModelMetadataProvider();
        }

        protected void Application_BeginRequest(object sender, EventArgs eventArgs)
        {
            if (Request.FilePath != null && Request.FilePath.EndsWith("/ErrorGeneration/BeginRequestException"))
            {
                throw new TestException("This exception was deliberately thrown by the BeginRequest handler method.");
            }
        }

        protected void Session_Start()
        {
            Session[SessionDataKey] = "The session data is available.";
        }

        public static string GetSessionData()
        {
            object sessionData = null;

            if (HttpContext.Current.Session != null)
            {
                sessionData = HttpContext.Current.Session[SessionDataKey];
            }

            return sessionData != null ? sessionData.ToString() : "The session data is missing.";
        }
    }
    // ReSharper restore InconsistentNaming
}