﻿using System.Reflection;
using System.Web.Mvc;

namespace MvcErrorHandling.MvcHelpers
{
    /// <summary>
    /// Add to an action method to ensure it is only called via AJAX requests.
    /// </summary>
    /// <remarks>
    /// Code by Stuart Leeks. Search for 'ASP.NET MVC: Partial rendering and AjaxAttribute'.
    /// </remarks>
    public class AjaxAttribute : ActionMethodSelectorAttribute
    {
        private readonly bool _ajax;

        public AjaxAttribute(bool ajax)
        {
            _ajax = ajax;
        }

        public override bool IsValidForRequest(ControllerContext controllerContext, MethodInfo methodInfo)
        {
            return _ajax == controllerContext.HttpContext.Request.IsAjaxRequest();
        }
    }
}