﻿using System;
using System.Configuration;
using MvcErrorHandling.DomainModel;
using MvcErrorHandling.DomainModel.AppSettingsModel;

namespace MvcErrorHandling.MvcHelpers
{
    /// <summary>
    /// A base class for all the common stuff in the ASPX error pages.
    /// </summary>
    public abstract class AspxErrorPageBase : System.Web.UI.Page
    {
        // ReSharper disable InconsistentNaming
        protected System.Web.UI.HtmlControls.HtmlForm form1;
        protected System.Web.UI.HtmlControls.HtmlGenericControl sessionLabel;
        protected System.Web.UI.HtmlControls.HtmlGenericControl exceptionLabel;
        protected System.Web.UI.HtmlControls.HtmlGenericControl aspxerrorpathLabel;
        // ReSharper restore InconsistentNaming

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            if (ConfigurationManager.AppSettings.Include(ErrorPageRenderingMode.ThrowException))
            {
                throw new TestException("Deliberately thrown while rendering the error page.");
            }

            if (ConfigurationManager.AppSettings.Include(TrySkipIisCustomErrorsMode.Skip))
            {
                Response.TrySkipIisCustomErrors = true;
            }

            sessionLabel.InnerText = MvcApplication.GetSessionData();
            exceptionLabel.InnerText = GetExceptionText();
            aspxerrorpathLabel.InnerText = GetAspxErrorPath();

            Server.ClearError();
        }

        private string GetExceptionText()
        {
            var exception = Server.GetLastError();

            return exception != null ? exception.Message : "The exception information is not available.";
        }

        private string GetAspxErrorPath()
        {
            var aspxErrorPath = Request.QueryString["aspxerrorpath"];

            return string.IsNullOrEmpty(aspxErrorPath) ? "The aspxerrorpath parameter is not available." : aspxErrorPath;
        }
    }
}