﻿using System;
using System.Collections.Generic;
using System.Web.Mvc;

namespace MvcErrorHandling.MvcHelpers
{
    /// <summary>
    /// A class derived from <see cref="DataAnnotationsModelMetadataProvider"/> that adds the behaviour of making 
    /// enum properties on view models be displayed using the 'Enum.cshtml' editor template.
    /// </summary>
    public class CustomDataAnnotationsModelMetadataProvider : DataAnnotationsModelMetadataProvider
    {
        protected override ModelMetadata CreateMetadata(
            IEnumerable<Attribute> attributes,
            Type containerType,
            Func<object> modelAccessor,
            Type modelType,
            string propertyName)
        {
            var metadata = base.CreateMetadata(attributes, containerType, modelAccessor, modelType, propertyName);

            if (metadata.ModelType.IsEnum)
            {     
                metadata.TemplateHint = "Enum";
            }

            return metadata;
        }
    }
}