﻿using System.Configuration;
using System.Web;
using MvcErrorHandling.DomainModel.AppSettingsModel;

namespace MvcErrorHandling.MvcHelpers
{
    /// <remarks>
    /// I needed to be able to control the HttpNotFoundHandler that is used in the View/Web.config file to restrict
    /// access to the Views directory and its contents. However, the System.Web.HttpNotFoundHandler class is marked
    /// as internal, so I couldn't just derive from it. Instead I recreated its behaviour as observed using 
    /// Telerik's JustDecompile program, adding in the extra behaviour I required.
    /// </remarks>
    public class CustomHttpNotFoundHandler : IHttpHandler
    {
        public bool IsReusable { get { return true; } }

        public void ProcessRequest(HttpContext context)
        {
            if (ConfigurationManager.AppSettings.Include(ManualNotFoundErrorHandlingMode.WithinMvcFramework))
            {
                NotFoundHandlerControllerFactory.CreateAndInvokeNotFoundController(new HttpContextWrapper(context));
            }
            else
            {
                throw new HttpException(404, "Thrown by CustomHttpNotFoundHandler");
            }
        }
    }
}