﻿using System;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Web.Mvc;
using System.Web.Mvc.Html;
using MvcErrorHandling.DomainModel;

namespace MvcErrorHandling.MvcHelpers
{
    public static class HtmlHelperExtensions
    {
        /// <remarks>
        /// This EnumDropDownListFor extension method is by stuartle; 
        /// Google 'Creating a DropDownList helper for enums'.
        /// </remarks>
        public static MvcHtmlString EnumDropDownListFor<TModel, TEnum>(
            this HtmlHelper<TModel> htmlHelper,
            Expression<Func<TModel, TEnum>> expression)
        {
            var metadata = ModelMetadata.FromLambdaExpression(expression, htmlHelper.ViewData);

            var values = Enum.GetValues(metadata.Model.GetType()).Cast<TEnum>();

            var selectListItems = values.Select(
                value => new SelectListItem
                             {
                                 Text = ToPrettyString(metadata.Model, value.ToString()),
                                 Value = value.ToString(),
                                 Selected = value.Equals(metadata.Model)
                             });

            return htmlHelper.DropDownListFor(expression, selectListItems);
        }

        private static string ToPrettyString(object enumObj, string enumValue)
        {
            var builder = new StringBuilder();

            foreach (var ch in enumValue)
            {
                if (char.IsUpper(ch) && builder.Length > 0)
                {
                    builder.Append(" ");
                }

                builder.Append(ch);
            }

            if (IsDefaultEnumValue(enumObj, enumValue))
            {
                builder.Append(" (Default)");
            }

            return builder.ToString();
        }

        private static bool IsDefaultEnumValue(object enumObj, string enumValue)
        {
            var attribute = enumObj.GetType().GetMember(enumValue).First()
                .GetCustomAttributes(typeof (DefaultEnumValueAttribute), false)
                .Cast<DefaultEnumValueAttribute>()
                .SingleOrDefault();

            return attribute != null;
        }
    }
}