﻿using System.Threading;
using System.Web;
using AutoMapper;
using MvcErrorHandling.DomainModel;

namespace MvcErrorHandling.Services
{
    public class ErrorConfigurationService
    {
        private static readonly object ConfigurationLock = new object();

        public ErrorConfigurationView ReadErrorConfiguration()
        {
            lock (ConfigurationLock)
            {
                var errorConfiguration = new ErrorConfiguration(GetVirtualPath());

                errorConfiguration.Read();

                var view = Mapper.Map<ErrorConfiguration, ErrorConfigurationView>(errorConfiguration);

                return view;
            }
        }

        public void WriteErrorConfiguration(ErrorConfigurationView view)
        {
            lock (ConfigurationLock)
            {
                var errorConfiguration = new ErrorConfiguration(GetVirtualPath());

                errorConfiguration.Read();
                                
                errorConfiguration.CustomErrors.UpdateSection(
                    view.CustomErrorsMode,
                    view.CustomErrorsRedirectMode,
                    view.CustomErrorsNotFoundErrorRedirect,
                    view.CustomErrorsForbiddenErrorRedirect,
                    view.CustomErrorsServerErrorRedirect);

                errorConfiguration.HttpRuntime.UpdateSection(
                    view.RelaxedUrlToFileSystemMappingMode);

                errorConfiguration.HttpErrors.UpdateSection(
                    view.HttpErrorsMode,
                    view.HttpErrorsExistingResponseMode,
                    view.HttpErrorsNotFoundErrorRedirect,
                    view.HttpErrorsForbiddenErrorRedirect,
                    view.HttpErrorsServerErrorRedirect);

                errorConfiguration.AppSettingsWriter.UpdateSettings(
                    view.ErrorTriggerLinkOpenMode,
                    view.ManualServerErrorHandlingMode,
                    view.TrySkipIisCustomErrorsMode,
                    view.ErrorPageRenderingMode,
                    view.ErrorPageExceptionsMode,
                    view.ManualNotFoundErrorHandlingMode,
                    view.ManualErrorsRenderingMode);

                errorConfiguration.Persist();

                Delay();
            }   
        }

        public void ResetErrorConfiguration()
        {
            lock (ConfigurationLock)
            {
                var errorConfiguration = new ErrorConfiguration(GetVirtualPath());

                errorConfiguration.Reset();
                errorConfiguration.Persist();

                Delay();
            }
        }

        // I'm erring on the side of caution by adding a delay between Web.config being updated 
        // and the response being received by the client.
        private static void Delay()
        {
            Thread.Sleep(3000);
        }

        private static string GetVirtualPath()
        {
            var virtualPath = HttpContext.Current.Request.ApplicationPath ?? string.Empty;

            virtualPath = virtualPath.Trim();

            if (!virtualPath.EndsWith("/"))
            {
                virtualPath = virtualPath + "/";
            }

            if (!virtualPath.StartsWith("/"))
            {
                virtualPath = "/" + virtualPath;
            }

            return virtualPath;
        }
    }
}