MvcErrorHandling
================

## Introduction ##

This repository contains an ASP.NET MVC Version 3 Web application that supports a series of posts that I wrote on [my blog](http://perspectivespace.com) about how to handle errors in such an application. The index to the series can be found [here](http://perspectivespace.com/error-handling-in-aspnet-mvc-3-index-of-posts).

If you have previously tried experimenting with the error handling features in ASP.NET and IIS, you will know that it is a chore to do so. To remedy this, I have created a simple ASP.NET MVC 3 demonstration Web application that can be dynamically configured as to how it handles errors.

## Requirements ##

- You need to be running a Windows OS that includes IIS version 7.0 or 7.5, such as Windows 7 or Windows Server 2008 R2.

- You need to have installed Microsoft Visual Studio 2010 and [ASP.NET MVC version 3](http://www.asp.net/mvc/mvc3). If you are using an Express version of Visual Studio, use Visual Web Developer 2010 Express.

- You need to be using a modern browser, such as a recent version of Chrome, Firefox or Internet Explorer.

## Installation ##

1. Get the Web application source code from the [project page](https://github.com/stevejay/MvcErrorHandling) at GitHub. You can download it as a zip file or you can use [Git](http://code.google.com/p/msysgit/) to clone the repository.

2. Run Visual Studio 2010 in Administrator mode and open the solution file MvcErrorHandling.sln. 

3. Build the solution by pressing Ctrl+Shift+B.

4. Right-click the application project in the Solution Explorer window and select the Properties option.

5. In the Properties window, select the Web tab and tick the Use Local IIS Web server option. Set the Project Url value to http://localhost/MvcErrorHandling.

6. Click the Create Virtual Directory button.

7. Enter the URL http://localhost/MvcErrorHandling in your browser. You should now be viewing the demo application.

If you want to check how the error handling features behave when the application is running under the Visual Studio Development Server, you can press Ctrl+F5 in Visual Studio 2010.

## A Quick Tour ##

The demo application consists of a single page that is split into two sections.

- The upper section is the Settings section and it consists of a number of Web.config file settings. To change these, set the values as required and then click the Update button. Notice that all settings have a default value. You can restore all the default values at once by clicking the Reset button.

- The lower section is the Error Trigger Links section and it contains a number of links that you can click to trigger particular errors.

When you click on an error link, you are going to be interested in three things:

- the page that is returned;

- its HTTP status code;

- whether or not a redirect occurred.

Out of the box, the demo application is configured to display the response in an iframe below the Error Trigger Links section. This is the behaviour when selecting Frame for the Link open mode setting. You can easily see the page that is returned and its status code, but you cannot see if a redirect occurred. If you instead select New Window for the Link open mode setting, the response will be displayed in a new tab. This allows you to see if a redirect occurred, as the URL displayed in that tab will be for the error page rather than the requested page.

You may prefer to use the browser developer tools to view the complete request time line and so view response codes and redirects in that way. In fact, I cannot get the response status code for the error links that perform a POST request and so you have to use those tools to view that value.

## Caveats ##

The demo application is unusual in that it edits its own Web.config file. When that file is updated, the server notices and reloads the site's application domain so that subsequent requests are handled according to the new state of Web.config. This process is quick; I am able to make a request right after updating Web.config and see the altered behaviour. This may not be true for all users, and I would be grateful for feedback if problems occur.

The application expects the sections of its Web.config file that relate to error handling and application settings to have particular values. If you were to manually edit the Web.config file, you could cause the application to be unable to parse the information. If this happens, just reload the home page and the application should reset itself.

Finally, when using the Frame option I have to issue two requests. The first is done so that I can get the response status code, while the second is done so that the response will be correctly displayed in the iframe. This is only something you need to be aware of if you start debugging the demo application.